import logging
import os

from setuptools import find_packages, setup

logger = logging.getLogger(__name__)

# Get the base directory
here = os.path.dirname(__file__)
if not here:
    here = os.path.curdir
here = os.path.abspath(here)

try:
    readme = os.path.join(here, "README.md")
    long_description = open(readme, "r").read()
except IOError:
    logger.warning("README file not found or unreadable.")
    long_description = "See https://github.com/brunns/brunns-row/"

setup(
    name="brunns-row",
    zip_safe=False,
    version="1.0.1",
    description="Convenience wrapper for DB API and csv.DictReader rows, and similar.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Simon Brunning",
    author_email="simon@brunningonline.net",
    url="https://github.com/brunns/brunns-row/",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    package_data={"": ["README.md"]},
    include_package_data=True,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Testing",
    ],
    python_requires=">=2.7",
    install_requires=[
        # "furl~=2.0",
        # "attrs~=18.0",
        # "more_itertools~=4.0",
        "pyhamcrest~=1.9",
        "pytest~=3.0",
        # 'enum34~=1.0;python_version<"3.4"',
        "six~=1.0",
        # "pendulum~=2.0",
        # "beautifulsoup4~=4.0",
        # "python-box~=3.2",
        # 'singledispatch~=3.4;python_version<"3.4"',
        # 'pathlib~=1.0;python_version<"3.4"',
        # 'functools32~=3.2;python_version<"3.2"',
    ],
)



# encoding=utf-8
from __future__ import absolute_import, division, print_function, unicode_literals

import collections
import logging

import six

logger = logging.getLogger(__name__)


class RowWrapper(object):
    """
    Build lightweight row tuples for DB API and csv.DictReader rows.

    Inspired by Greg Stein's lovely dtuple module,
    https://code.activestate.com/recipes/81252-using-dtuple-for-flexible-query-result-access,
    which I can't find online any longer, isn't on pypi, and doesn't support Python 3 without some fixes.

    Initializer takes a sequence of column descriptions, either names, or tuples of names and other metadata (which
    will be ignored). For instance, it's happy to take a DB API cursor description, or a csv.DictReader's fieldnames
    property. Provides a wrap(row) method for wrapping rows.

    Some characters which are illegal in identifiers will be replaced when building the row tuples - currently "-" and
    " " characters will be replaced with "_"s.

    >>> cursor = conn.cursor()
    >>> cursor.execute("SELECT kind, rating FROM sausages ORDER BY rating DESC;")
    >>> wrapper = RowWrapper(cursor.description)
    >>> rows = [wrapper.wrap(row) for row in cursor.fetchall()]

    >>> reader = csv.DictReader(csv_file)
    >>> wrapper = RowWrapper(reader.fieldnames)
    >>> rows = [wrapper.wrap(row) for row in reader]
    """

    def __init__(self, description):
        self.names = (
            [col for col in description]
            if isinstance(description[0], six.string_types)
            else [col[0] for col in description]
        )
        self.namedtuple = collections.namedtuple("RowTuple", [self._id_fix(n) for n in self.names], rename=True)

    @staticmethod
    def _id_fix(name):
        for old, new in [("-", "_"), (" ", "_")]:
            name = name.replace(old, new)
        return name

    def wrap(self, row):
        """Return row tuple for row."""
        return (
            self.namedtuple(**{self._id_fix(k): row[k] for k in self.names})
            if isinstance(row, collections.Mapping)
            else self.namedtuple(**{self._id_fix(n): r for n, r in zip(self.names, row)})
        )

    def wrap_all(self, rows):
        """Return row tuple for each row in rows."""
        return (self.wrap(r) for r in rows)

    def __call__(self, row):
        return self.wrap(row)
